==> instabug-core-14.2.1-sources.jar: com/instabug/library/screenshot/instacapture/HierarchyExtractor.kt <==
package com.instabug.library.screenshot.instacapture

import android.app.Activity
import android.graphics.Rect
import android.view.View
import android.view.WindowManager
import com.instabug.library.instacapture.screenshot.FieldHelper.getRootViews
import com.instabug.library.instacapture.screenshot.windowType
import com.instabug.library.interactionstracking.IBGUINode
import com.instabug.library.interactionstracking.UINodeTransformer
import com.instabug.library.internal.LazyProvider
import com.instabug.library.internal.servicelocator.CoreServiceLocator
import com.instabug.library.internal.servicelocator.CoreServiceLocator.ignoredViewsIds
import com.instabug.library.settings.SettingsManager
import com.instabug.library.util.threading.PoolProvider
import java.util.concurrent.Future

/**
 * extracts views rectangles in the provided activity by iterating on the root views
 * and extract views location and map it to rect
 * */
interface HierarchyExtractor {

    fun extract(activity: Activity): Future<List<Rect>>

    companion object Factory {
        fun create(
            viewsFilter: ViewsFilter = CoreServiceLocator.composedViewsFilter,
            nodeTransformerProvider: LazyProvider<UINodeTransformer<View>> =
                LazyProvider { CoreServiceLocator.uiNodesTransformer }
        ): HierarchyExtractor =
            HierarchyExtractorImpl(viewsFilter, nodeTransformerProvider)
    }
}

class HierarchyExtractorImpl(
    private val viewsFilter: ViewsFilter,
    private val nodeTransformerProvider: LazyProvider<UINodeTransformer<View>>
) : HierarchyExtractor {
    override fun extract(activity: Activity): Future<List<Rect>> = PoolProvider.submitIOTask {
        val rects = mutableListOf<Rect>()
        val ignoredViews = ignoredViewsIds
        activity.window?.peekDecorView()
            ?.let { nodeTransformerProvider().transform(it) }
            ?.also { traverseAndExtractRects(it, rects, ignoredViews) }

        val shouldMaskDialogs = SettingsManager.getInstance().shouldCaptureDialog
        getRootViews(activity, ignoredViews)
            .filterNot { root -> root.isOverlayViews && !shouldMaskDialogs }
            .filterNot { root -> root.windowType == WindowManager.LayoutParams.TYPE_BASE_APPLICATION }
            .map { root -> nodeTransformerProvider().transform(root.view) }
            .forEach { node -> traverseAndExtractRects(node, rects, ignoredViews) }
        rects
    }

    private fun traverseAndExtractRects(
        node: IBGUINode,
        rects: MutableList<Rect>,
        ignoredViews: IntArray
    ) {
        if (node.isIBGComponent(ignoredViews)) return
        if (viewsFilter shouldMask node) {
            node.asRect()?.also(rects::add)
            return
        }
        for (index in 0 until node.childCount)
            node.getChildAt(index)
                ?.let { child -> traverseAndExtractRects(child, rects, ignoredViews) }
    }
}

==> instabug-core-16.0.2-sources.jar: com/instabug/library/screenshot/instacapture/HierarchyExtractor.kt <==
package com.instabug.library.screenshot.instacapture

import android.app.Activity
import android.graphics.Point
import android.graphics.Rect
import android.view.View
import android.view.WindowManager
import com.instabug.library.instacapture.screenshot.FieldHelper.getRootViews
import com.instabug.library.instacapture.screenshot.windowType
import com.instabug.library.interactionstracking.IBGUINode
import com.instabug.library.interactionstracking.UINodeTransformer
import com.instabug.library.internal.LazyProvider
import com.instabug.library.internal.servicelocator.CoreServiceLocator
import com.instabug.library.internal.servicelocator.CoreServiceLocator.ignoredViewsIds
import com.instabug.library.settings.SettingsManager
import com.instabug.library.util.threading.PoolProvider
import java.util.concurrent.Future

/**
 * extracts views rectangles in the provided activity by iterating on the root views
 * and extract views location and map it to rect
 * */
interface HierarchyExtractor {

    fun extract(activity: Activity): Future<List<Rect>>

    companion object Factory {
        fun create(
            viewsFilter: ViewsFilter = CoreServiceLocator.composedViewsFilter,
            nodeTransformerProvider: LazyProvider<UINodeTransformer<View>> =
                LazyProvider { CoreServiceLocator.uiNodesTransformer }
        ): HierarchyExtractor =
            HierarchyExtractorImpl(viewsFilter, nodeTransformerProvider)
    }
}

class HierarchyExtractorImpl(
    private val viewsFilter: ViewsFilter,
    private val nodeTransformerProvider: LazyProvider<UINodeTransformer<View>>
) : HierarchyExtractor {
    override fun extract(activity: Activity): Future<List<Rect>> = PoolProvider.submitIOTask {
        val rects = mutableListOf<Rect>()
        val ignoredViews = ignoredViewsIds
        activity.window?.peekDecorView()
            ?.let { nodeTransformerProvider().transform(it) }
            ?.also { traverseAndExtractRects(it, rects, ignoredViews, Point(0, 0)) }

        val shouldMaskDialogs = SettingsManager.getInstance().shouldCaptureDialog
        getRootViews(activity, ignoredViews)
            .filterNot { root -> root.isOverlayViews && !shouldMaskDialogs }
            .filterNot { root -> root.windowType == WindowManager.LayoutParams.TYPE_BASE_APPLICATION }
            .forEach { root ->
                val node = nodeTransformerProvider().transform(root.view)
                traverseAndExtractRects(node, rects, ignoredViews, Point(root.left, root.top))
            }
        rects
    }

    private fun traverseAndExtractRects(
        node: IBGUINode,
        rects: MutableList<Rect>,
        ignoredViews: IntArray,
        translate: Point
    ) {
        if (node.isIBGComponent(ignoredViews)) return
        if (viewsFilter shouldMask node) {
            node.asRect()
                // Translating the produced Rect with the amount provided by the root view.
                // Typically for compose & legacy Views dialogs, because retrieving a Rect per view
                // doesn't count for the actual coordinates of the whole dialog on the screen
                ?.apply { translate(translate) }
                ?.also(rects::add)
            return
        }
        for (index in 0 until node.childCount)
            node.getChildAt(index)
                ?.let { child -> traverseAndExtractRects(child, rects, ignoredViews, translate) }
    }

    private fun Rect.translate(amount: Point) {
        set(left + amount.x, top + amount.y, right + amount.x, bottom + amount.y)
    }
}
